import 'package:bones_api/bones_api.dart';
import 'package:bones_api/bones_api_adapter_postgre.dart';

import 'api_entities.dart';

class APIEntityRepositoryProvider extends EntityRepositoryProvider {
  static final APIEntityRepositoryProvider _instance =
      APIEntityRepositoryProvider._();

  factory APIEntityRepositoryProvider() => _instance;

  APIEntityRepositoryProvider._() {
    var apiConfig = apiRoot?.apiConfig;

    var postgreAdapter = PostgreSQLAdapter.fromConfig(
      apiConfig?['postgres'],
      defaultDatabase: 'postgres',
      defaultUsername: 'postgres',
      parentRepositoryProvider: this,
    );

    SQLEntityRepository<Address>(
        postgreAdapter, 'address', Address$reflection().entityHandler);

    SQLEntityRepository<Account>(
        postgreAdapter, 'account', Account$reflection().entityHandler);
  }

  APIRoot? get apiRoot => APIRoot.getWithinName('korthex', lastAsDefault: true);
}

class AddressAPIRepository extends APIRepository<Address> {
  AddressAPIRepository() : super(provider: APIEntityRepositoryProvider());

  FutureOr<Iterable<Address>> selectByState(String state) {
    return selectByQuery(' state == ? ', parameters: {'state': state});
  }
}

class AccountAPIRepository extends APIRepository<Account> {
  AccountAPIRepository() : super(provider: APIEntityRepositoryProvider());

  FutureOr<Iterable<Account>> selectAccountByEmail(String email) {
    return selectByQuery(' email == ? ',
        parameters: {'email': email}, limit: 1);
  }

  FutureOr<Iterable<Account>> selectAccountByAddressState(String state) {
    return selectByQuery(' address.state == ? ', parameters: [state]);
  }
}
